{% extends "base.html" %}
{% block title %}<h2>{{ task }}</h2>{% endblock title %}
{% block content %}
<code>{{ description }}</code>
<table role="grid">
    <thead>
        <tr>
            <th>Submitted at</th>
            <th>Status</th>
            <th>Severity</th>
            <th>Context</th>
        </tr>
    </thead>
    <tbody>
        {% for result in results %}
        <tr id="{{ result.id }}">
            <td>{{ result.submitted_at }}</td>
            <td>{{ result.status }}</td>
            <td>{{ result.severity }}</td>
            <td>{{ result.context }}</td>
        </tr>
        {% endfor %}
    </tbody>
</table>
{% endblock content %}
